# (det_id, run_n): (du, floor)
RUNDIC = {
    ( KM3NeT_00000014, 3816 ): (1, 18), #died between 3786 and 3816
    ( KM3NeT_00000014, 5103 ): (2, 1 ), #died on 5102 or 5103 https://elog.km3net.de/Operations+IT/3375
    ( KM3NeT_00000014, 5047 ): (2, 16), #died 5047 https://elog.km3net.de/Operations+IT/3326
    ( KM3NeT_00000043, 4227 ): (1, 1 )  #died 4227 https://elog.km3net.de/Operations+FR/1538
}
    #( KM3NeT_00000045, 0    ): (6, 6),  #died at deployment
    #( KM3NeT_00000045, 0    ): (6, 7),  #died at deployment
    #( KM3NeT_00000044, 0    ): (2, 15), #died at deployment https://elog.km3net.de/Operations+FR/2372
    #( KM3NeT_00000044, 0    ): (2, 16)  #died at deployment https://elog.km3net.de/Operations+FR/2372
    
